﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Drawing2D;

namespace howto_find_equilateral_triangles
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // The points.
        private PointF[] Points;

        // The solutions.
        private List<int[]> Solutions = new List<int[]>();

        // The solution we should display.
        private int CurrentSolution = 100;

        // Define the points and solutions.
        private void Form1_Load(object sender, EventArgs e)
        {
            DoubleBuffered = true;

            // Define the points.
            float dy = ClientSize.Height / 4;
            float dx = (float)(dy / Math.Sqrt(3));
            float top_x = ClientSize.Width / 2;
            float top_y = -dy / 2;
            Points = new PointF[]
            {
                new PointF(top_x - dx, top_y + dy),
                new PointF(top_x + dx, top_y + dy),
                new PointF(top_x - 2 * dx, top_y + 2 * dy),
                new PointF(top_x - 0 * dx, top_y + 2 * dy),
                new PointF(top_x + 2 * dx, top_y + 2 * dy),
                new PointF(top_x - 3 * dx, top_y + 3 * dy),
                new PointF(top_x - 1 * dx, top_y + 3 * dy),
                new PointF(top_x + 1 * dx, top_y + 3 * dy),
                new PointF(top_x + 3 * dx, top_y + 3 * dy),
                new PointF(top_x - 2 * dx, top_y + 4 * dy),
                new PointF(top_x - 0 * dx, top_y + 4 * dy),
                new PointF(top_x + 2 * dx, top_y + 4 * dy),
            };
        }

        // Find the solutions.
        private void btnFindSolutions_Click(object sender, EventArgs e)
        {
            Cursor = Cursors.WaitCursor;

            Solutions = new List<int[]>();
            for (int i = 0; i < Points.Length; i++)
            {
                for (int j = i + 1; j < Points.Length; j++)
                {
                    double dx_ij = Points[j].X - Points[i].X;
                    double dy_ij = Points[j].Y - Points[i].Y;
                    double dist_ij = Math.Sqrt(dx_ij * dx_ij + dy_ij * dy_ij);
                    for (int k = j + 1; k < Points.Length; k++)
                    {
                        double dx_jk = Points[k].X - Points[j].X;
                        double dy_jk = Points[k].Y - Points[j].Y;
                        double dist_jk = Math.Sqrt(dx_jk * dx_jk + dy_jk * dy_jk);
                        const double tiny = 0.0001;
                        if (Math.Abs(dist_ij - dist_jk) < tiny)
                        {
                            double dx_ki = Points[i].X - Points[k].X;
                            double dy_ki = Points[i].Y - Points[k].Y;
                            double dist_ki = Math.Sqrt(dx_ki * dx_ki + dy_ki * dy_ki);
                            if (Math.Abs(dist_jk - dist_ki) < tiny)
                            {
                                // This is a solution.
                                Solutions.Add(new int[] { i, j, k });
                            }
                        }
                    }
                }
            }

            lblNumSolutions.Text = Solutions.Count.ToString() + " solutions";
            btnFindSolutions.Enabled = false;
            btnShowSolutions.Enabled = true;
            btnShowAllSolutions.Enabled = true;
            Refresh();
            Cursor = Cursors.Default;
        }

        // Draw the circles and any triangles currently defined.
        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;

            // Draw the points.
            const float radius = 5;
            foreach (PointF pt in Points)
            {
                e.Graphics.FillEllipse(Brushes.Blue,
                    pt.X - radius, pt.Y - radius,
                    2 * radius, 2 * radius);
            }

            // Draw the current solution.
            if (CurrentSolution < 0)
            {
                // Draw all solutions.
                for (int i = 0; i < Solutions.Count; i++)
                {
                    DrawSolution(e.Graphics, i);
                }
            }
            else
            {
                // Draw the current solution.
                DrawSolution(e.Graphics, CurrentSolution);
            }
        }

        // Draw a solution.
        private void DrawSolution(Graphics gr, int solution_num)
        {
            if (solution_num >= Solutions.Count) return;

            // Make a list of the points in this solution.
            List<PointF> pts = new List<PointF>();
            foreach (int i in Solutions[solution_num]) pts.Add(Points[i]);
            gr.DrawPolygon(Pens.Red, pts.ToArray());
        }

        // Start showing solutions.
        private void btnShowSolutions_Click(object sender, EventArgs e)
        {
            // Disable the button.
            btnShowSolutions.Enabled = false;

            // Start at the first solution.
            CurrentSolution = 0;

            // Start the timer.
            tmrChangeSolution.Enabled = true;

            // Redraw.
            Refresh();
        }

        // Show the next solution.
        private void tmrChangeSolution_Tick(object sender, EventArgs e)
        {
            // Increment CurrentSolution. If the result is greater than the
            // last solution's index, disable the timer.
            tmrChangeSolution.Enabled = (++CurrentSolution < Solutions.Count);

            // If we're done drawing, enable the button.
            btnShowSolutions.Enabled = (!tmrChangeSolution.Enabled);

            // Redraw.
            Refresh();
        }

        // Show all of the solutions.
        private void btnShowAllSolutions_Click(object sender, EventArgs e)
        {
            CurrentSolution = -1;
            tmrChangeSolution.Enabled = false;
            btnShowSolutions.Enabled = true;
            Refresh();
        }
    }
}
